==> jsdom-5.6.1.tgz: package/lib/jsdom/living/parent-node.js <==
"use strict";

const defineGetter = require("../utils").defineGetter;

module.exports = function (core) {
  // https://dom.spec.whatwg.org/#interface-parentnode
  // Note that ParentNode is a "NoInterfaceObject"

  const implementers = [core.Document, core.DocumentFragment, core.Element];
  const ELEMENT_NODE = core.Node.ELEMENT_NODE;

  implementers.forEach(function (Constructor) {
    defineGetter(Constructor.prototype, "children", function () {
      if (!this._childrenList) {
        const self = this;
        this._childrenList = new core.HTMLCollection(this, function () {
          return self._childNodes.filter(function (node) {
            return node.nodeType === ELEMENT_NODE;
          });
        });
      }
      this._childrenList._update();
      return this._childrenList;
    });

    defineGetter(Constructor.prototype, "firstElementChild", function () {
      for (let i = 0; i < this._childNodes.length; ++i) {
        const child = this._childNodes[i];
        if (child.nodeType === ELEMENT_NODE) {
          return child;
        }
      }

      return null;
    });

    defineGetter(Constructor.prototype, "lastElementChild", function () {
      for (let i = this._childNodes.length - 1; i >= 0; --i) {
        const child = this._childNodes[i];
        if (child.nodeType === ELEMENT_NODE) {
          return child;
        }
      }

      return null;
    });

    defineGetter(Constructor.prototype, "childElementCount", function () {
      return this.children.length;
    });
  });
};

==> jsdom-6.0.1.tgz: package/lib/jsdom/living/parent-node.js <==
"use strict";

const defineGetter = require("../utils").defineGetter;
const domSymbolTree = require("../living/helpers/internal-constants").domSymbolTree;
const NODE_TYPE = require("../living/node-type");

module.exports = function (core) {
  // https://dom.spec.whatwg.org/#interface-parentnode
  // Note that ParentNode is a "NoInterfaceObject"

  const implementers = [core.Document, core.DocumentFragment, core.Element];

  implementers.forEach(function (Constructor) {
    defineGetter(Constructor.prototype, "children", function () {
      if (!this._childrenList) {
        const self = this;
        this._childrenList = new core.HTMLCollection(this, function () {
          return domSymbolTree.childrenToArray(self, {filter: function (node) {
            return node.nodeType === NODE_TYPE.ELEMENT_NODE;
          }});
        });
      }
      this._childrenList._update();
      return this._childrenList;
    });

    defineGetter(Constructor.prototype, "firstElementChild", function () {
      for (const child of domSymbolTree.childrenIterator(this)) {
        if (child.nodeType === NODE_TYPE.ELEMENT_NODE) {
          return child;
        }
      }

      return null;
    });

    defineGetter(Constructor.prototype, "lastElementChild", function () {
      for (const child of domSymbolTree.childrenIterator(this, {reverse: true})) {
        if (child.nodeType === NODE_TYPE.ELEMENT_NODE) {
          return child;
        }
      }

      return null;
    });

    defineGetter(Constructor.prototype, "childElementCount", function () {
      return this.children.length;
    });
  });
};
